package com.corejdo.casestudy.usecase;

import com.corejdo.casestudy.model.User;
import com.corejdo.casestudy.tools.CmdLine;

import javax.jdo.*;
import java.util.*;
/**
 * This is the abstract base class of all use cases.
 * It performs the simple operations by
 * creating a PersistenceManager, starting a transaction, and
 * committing/aborting it on success or failure. The sub-classes
 * of such use case only have to implement the <tt>_execute()</tt> method.
 *
 * @author Heiko Bobzin, Keiron McCammon, Sameer Tyagi, Michael Vorburger
 * @version 1.0
 */

public abstract class AbstractUseCase
{
    private Context context;

    public AbstractUseCase() {
    }

    final Context getContext()
    {
        if (context == null) {
            throw new RuntimeException("No context set.");
        }
        return context;
    }

    final void setContext(Context context)
    {
        this.context = context;
    }

    final PersistenceManager getPM()
    {
        return getContext().getPM();
    }

    /**
     * This method creates a transaction context, starts the transaction
     * and commits all work, after the <tt>_exceute</tt> method has been
     * successfully invoked. If the sub-class throws an exception, the
     * transaction is aborted.
     */
    public final void execute()
    {
        if (!isAllowed()) throw new UseCaseException("Not allowed.");
        getPM().currentTransaction().begin();
        boolean active = true;
        try {
            _execute();
            getPM().currentTransaction().commit();
            active = false;
        }
        finally {
            if (active) {
                try {
                    getPM().currentTransaction().rollback();
                    active = false;
                }
                catch (JDOUserException ex)
                {
                    ex.printStackTrace();
                }
            }
        }
    }

    /**
     * User security check. In derived classes this method is overridden
     * and returns false, if access is restricted.
     */
    public boolean isAllowed()
    {
        return true;
    }

    /**
     * Implement the basic operation of the use case in this method.
     * It is called by the <tt>excecute()</tt> method in the context of a running transaction
     * context.
     */
    protected abstract void _execute();

    /**
     * Command line version of the use case.
     */
    protected abstract void run(String args[]);

    /**
     * This method is called by the sub-class main method
     * to intialize the command line tool.
     * @param args
     */
    void runCmdLine(String args[])
    {
        this.context = Main.mainContext;
        CmdLine.init(args);
        run(args);
    }

    final User getUser()
    {
        User user = this.getContext().user;
        if (user == null) {
            throw new UseCaseException("You must login first.");
        }
        return user;
    }

}
