package com.corejdo.casestudy.usecase;

import com.corejdo.casestudy.model.User;
import com.corejdo.casestudy.model.Rights;

import javax.jdo.*;
import java.util.*;
import java.io.*;
/**
 * This class provides context information common to all use cases.
 * It includes a current persistence manager, the library singleton,
 * a user session and other common information.
 *
 * In the servlet case, the ServletHelper class contains a valid context as
 * long as the JSP/Servlet session lives. In the command line version, only
 * a single context is created.
 *
 * @author Heiko Bobzin, Keiron McCammon, Sameer Tyagi, Michael Vorburger
 * @version 1.0
 */

public final class Context
{
    private PersistenceManager currentPersistenceManager;
    private static PersistenceManagerFactory pmf;

    // These are context/session wide attributes:
    User user;
    Rights userRights = new Rights();

    Context()
    {
    }
    /**
     * Creates or returns a JDO factory. The properties are loaded
     * from a file.
     */
    private PersistenceManagerFactory getPMF()
    {
	    if (pmf == null) {

	      String filename =
	        System.getProperty("jdo.properties");

	      if (filename == null) {

	        throw new JDOFatalUserException(
	          "System property 'jdo.properties' not defined");
	      }

	      Properties properties = new Properties();

	      try {

	        FileInputStream file = 
	          new FileInputStream(filename);

	        properties.load(file);

	        file.close();

	        pmf = JDOHelper.getPersistenceManagerFactory(
	          properties);
	      }

	      catch (java.io.IOException e) {

	        throw new JDOFatalUserException(
	          "Error reading '" + filename + "'",
	          e);
	      }
	    }
	    return pmf;
    }

    /**
     * Creates or returns a PersistenceManager.
     * If a servlet helper is associated with the current use case,
     * a persistence manager may be taken from the helper object.
     */
    PersistenceManager getPM()
    {
        if (currentPersistenceManager == null) {
            currentPersistenceManager = getPMF().getPersistenceManager();
        }
        return currentPersistenceManager;
    }

    /**
     * Called by outside application management.
     *
     * In case of servlets, this method is called on session timeout.
     * The command line version calls this method at system exit.
     */
    void close()
    {
        if (currentPersistenceManager != null) {
            currentPersistenceManager.close();
            currentPersistenceManager = null;
        }
    }

    public boolean isLoggedIn()
    {
        return user != null;
    }
}