package com.corejdo.casestudy.usecase;

import com.corejdo.casestudy.tools.CmdLine;
import com.corejdo.casestudy.model.*;
import javax.jdo.*;
import java.util.*;

/**
 * This class lists books.
 *
 * @author Heiko Bobzin, Keiron McCammon, Sameer Tyagi, Michael Vorburger
 * @version 1.0
 */

public class ListBooks
    extends BookOperation
{
    int startNumber = 0;
    int count = 10;
    List result;

    String filterAuthor = "";
    Query query;
    Extent extent;

    /**
     * Sets the start offset. Since JDO does not support a "seek" operation on
     * extents, we implemented our own by simply calling iterator.next().
     * @param i
     */
    public void setStartNumber(int i)
    {
        startNumber = i;
    }

    /**
     * Sets the number of entries returned by this operation.
     * @param i number of entries
     */
    public void setCount(int i)
    {
        count = i;
    }

    /**
     * This data object is used to return the results.
     * @author Heiko Bobzin, Keiron McCammon, Sameer Tyagi, Michael Vorburger
     * @version 1.0
     */
    public class BookDTO
    {
        public final String title;
        public final String copyright;
        public final String ISBN;
        public final String author;

        BookDTO(Book b)
        {
            this.title = b.title;
            this.copyright = b.copyright;
            this.ISBN = b.ISBN;
            this.author = b.author;
        }

        public String toString()
        {
            return   "Title:  "+title+
                   "\n   Author: "+author+
                   "\n   ISBN:   "+ISBN+
                   "\n   (C):    "+copyright;
        }
    }

    /**
     * Iterates over elements of the book Extent.
     * Since JDO does not provide a "seek" operation, we implemented it
     * by simply calling iterator.next() up to the start position.
     */
    protected void _execute()
    {
        result = new ArrayList();
        extent = getPM().getExtent(Book.class,true);
        query = null;
        Iterator iter = filter(extent);
        int i = this.startNumber;
        while (i-- > 0 && iter.hasNext()) iter.next();
        i = this.count;
        while (i-- > 0 && iter.hasNext()) {
            result.add(new BookDTO((Book)iter.next()));
        }
        if (query != null) {
            query.close(iter);
        } else {
            extent.close(iter);
        }
    }

    protected Iterator filter(Extent extent)
    {
        if (filterAuthor != null && filterAuthor.length() > 0) {
            query = getPM().newQuery(Book.class, "author == val");
            query.declareParameters("java.lang.String val");
            query.setCandidates(extent);
            Collection result = (Collection)query.execute(filterAuthor);
            return result.iterator();
        } else {
            return extent.iterator();
        }
    }

    /**
     * Command line version.
     */
    public static void main(String[] args) {
        new ListBooks().runCmdLine(args);
    }

    protected void run(String[] args) {
        setStartNumber(CmdLine.getIntVar("Start at",0));
        setCount(CmdLine.getIntVar("Count",10));
        execute();
        System.out.println("Result: ");
        Iterator iter = getResult();
        int i = 1;
        while (iter.hasNext()) {
            System.out.println(""+i+": "+iter.next());
            i++;
        }
    }

    /**
     * The resulting iterator returns BookDTO objects.
     */
    public Iterator getResult()
    {
        return result.iterator();
    }
    public int getCount()
    {
        return count;
    }
    public int getStartNumber()
    {
        return startNumber;
    }
    public String getFilterAuthor()
    {
        return filterAuthor;
    }
    public void setFilterAuthor(String filterAuthor)
    {
        this.filterAuthor = filterAuthor;
    }
}
