package com.corejdo.examples.chapter3;

import java.util.Properties;
import javax.jdo.*;

import com.corejdo.examples.util.PersistenceManagerFactoryHelper;
import com.corejdo.examples.model.Author;

/**
 * <p>This is a simple example to show how to create and persist 
 * a single instance of a persistence capable class within a
 * transaction and use a try/finally block to ensure that any active
 * transaction is aborted and the PersistenceManager is closed
 * even if an unhandled exception is thrown.</p>
 * <p>It shows how to use <code>PersistenceManagerFactory</code>
 * to get a <code>PersistenceManager</code> instance. How to get
 * the current <code>Transaction</code> and use it to begin and
 * commit a transaction and how to use <code>makePersistent()</code>
 * to make an instance persistent. It uses a finally block to ensure
 * that even if an exception is thrown, if there is an active transaction 
 * it is aborted and the <code>PersistenceManager</code> instance 
 * is closed.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined. After
 * the example has been run use the appropriate database browsing tools
 * to confirm the <code>Author</code> instance has been persisted.</p> 
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 *
 */
public final class CreateWithExceptionsExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = null;

    try {

      pm = pmf.getPersistenceManager();

      Transaction tx = pm.currentTransaction();

      tx.begin();

      Author author = new Author("Keiron McCammon");

      pm.makePersistent(author);

      tx.commit();

      pm.close();
    }

    finally {

      if (pm != null && !pm.isClosed()) {

        if (pm.currentTransaction().isActive()) {

          pm.currentTransaction().rollback();
        }

        pm.close();
      }
    }
  }

  private CreateWithExceptionsExample() {}
}
