package com.corejdo.examples.chapter3;

import java.util.Properties;
import javax.jdo.*;

import com.corejdo.examples.util.PersistenceManagerFactoryHelper;
import com.corejdo.examples.model.Author;
import com.corejdo.examples.model.Book;

/**
 * <p>This is a simple example to show how persistence by reachability
 * ensures that instances of a persistence capable class referenced by
 * another instance that is made persistent also become persistent.</p>
 * <p>It creates an <code>Author</code> instance and a
 * <code>Book</code> instance. The <code>Author</code>
 * instance references the <code>Book</code> instance.
 * Only the <code>Author</code> instance is actually 
 * made persistent, however the <code>Book</code> instance is also 
 * made persistent because it is referenced by the <code>Author</code>
 * instance.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> and 
 * <code>Book</code> classes defined.
 * After the example has been run use the appropriate database browsing tools
 * to confirm that both the <code>Book</code> and <code>Author</code>
 * instances have been persisted.</p> 
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 * @see com.corejdo.examples.model.Book
 *
 */
public final class CreateWithReachabilityExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();

    tx.begin();

    Author author = new Author("Keiron McCammon");

    Book book = new Book("Core Java Data Objects", "0-13-140731-7");

    author.addBook(book);
    
    pm.makePersistent(author);

    tx.commit();

    pm.close();
  }
  
  private CreateWithReachabilityExample() {}
}
