package com.corejdo.examples.chapter3;

import java.util.*;
import javax.jdo.*;

import com.corejdo.examples.util.PersistenceManagerFactoryHelper;
import com.corejdo.examples.model.Author;

/**
 * <p>This simple example shows how the same persistent object can be
 * retrieved by two seperate <code>PersistenceManager</code> instances,
 * which results in there being two seperate instances in-memory.
 * However the JDO object identity of the two instances is compared
 * to show that the instances actually represent the same persistent object.</p>
 * <p>An <code>Author</code> instance is created using one
 * <code>PersistenceManager</code> instance. Another
 * <code>PersistenceManager</code> instance is used to execute a 
 * <code>Query</code> to find the <code>Author</code> instance previously created.
 * The two in-memory instances are compared to prove they are different 
 * Java objects. However, their JDO object identities are compared
 * to prove the instances do represent the same persistent object
 * in the datastore.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.</p> 
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 * 
 */
public final class ObjectIdentityExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    /*
     * Get the first PersistenceManager instance and create a new
     * Author instance
     */
    PersistenceManager pm1 = pmf.getPersistenceManager();

    Transaction tx1 = pm1.currentTransaction();
    
    tx1.begin();
    
    Author author1 = new Author("Keiron McCammon");
    
    pm1.makePersistent(author1);
    
    tx1.commit();
    
    /*
     * Get the second PersistenceManager instance can retrieve the
     * Author instance previously created
     */
    PersistenceManager pm2 = pmf.getPersistenceManager();
        
    Transaction tx2 = pm2.currentTransaction();
    
    tx2.begin();
    
    Query query =
      pm2.newQuery(Author.class, "name == \"Keiron McCammon\"");
    
    Collection authors = (Collection) query.execute();
    
    Author author2 = (Author) authors.iterator().next();
    
    query.close(authors);
    
    tx2.commit();
    
    /*
     * Confirm that the two in-memory instances are different
     * Java objects
     */
    if (author1 != author2)
      System.out.println("There are multiple objects in memory.");
      
    Object emp1Id = JDOHelper.getObjectId(author1);    
    Object emp2Id = JDOHelper.getObjectId(author2);

    /*
     * Confirm that the two in-memory instances actually represent
     * the same persistent object in the datastore
     */    
    if (emp1Id.equals(emp2Id))
      System.out.println("But they represent the same Author.");
    
    pm1.close();
    pm2.close();
  }
  
  private ObjectIdentityExample() {}
}
