package com.corejdo.examples.chapter3;

import java.util.Properties;
import javax.jdo.*;

import com.corejdo.examples.util.PersistenceManagerFactoryHelper;
import com.corejdo.examples.model.Author;

/**
 * <p>This simple example creates a new <code>Author</code> instance and
 * makes it persistent. In another transaction it uses the reference to
 * the previously created <code>Author</code> instance and retrieves it
 * from the datastore in the new transaction.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.</p>
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 *
 */
public final class ReadByNavigationExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main ( String[] args ) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();

    /*
     * Create a new Author instance and make it persistent
     */    
    tx.begin();
    
    Author author = new Author("Keiron McCammon");

    pm.makePersistent(author);
    
    tx.commit();

    /*
     * The fields of the Author instance are no longer in-memory
     * after the transaction ends, but the reference is still valid
     */
     
    tx.begin();

    /*
     * Accessing the Author instance in another transaction will
     * transparently retrieve it again from the datastore
     */   
    String name = author.getName();
    
    System.out.println("Author's name is '" + name + "'.");
    
    tx.commit();
    
    pm.close();
  }
  
  private ReadByNavigationExample() {}
}
