package com.corejdo.examples.chapter3;

import java.util.*;
import javax.jdo.*;

import com.corejdo.examples.util.PersistenceManagerFactoryHelper;
import com.corejdo.examples.model.Author;

/**
 * <p>This simple example uses a <code>Query</code> to find a previously
 * created <code>Author</code> instance.</p>
 * <p>It uses a <code>PersistenceManager</code> to create a <code>Query</code>
 * on the <code>Author</code> class. The filter will match any instances that
 * have the name "Keiron McCammon". The <code>Query</code> is executed and
 * the name of the first instance in the returned <code>Collection</code>
 * printed out to validate its name is indeed "Keiron McCammon".</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.
 * At least one <code>Author</code> instance needs to already exist in the
 * database with the name "Keiron McCammon", this can be created using
 * {@link com.corejdo.examples.chapter3.CreateExample CreateExample}.</p>
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 * @see com.corejdo.examples.chapter3.CreateExample
 *
 */
public final class ReadByQueryExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main ( String[] args ) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();
    
    tx.begin();
    
    Query query = pm.newQuery(Author.class, "name == \"Keiron McCammon\"");
    
    Collection result = (Collection) query.execute();
    
    Author author = (Author) result.iterator().next();
    
    String name = author.getName();
    
    System.out.println("Author's name is '" + name + "'.");
    
    query.close(result);
    
    tx.commit();
    
    pm.close();
  }
  
  private ReadByQueryExample() {}
}
