package com.corejdo.examples.chapter3;

import java.util.*;
import javax.jdo.*;

import com.corejdo.examples.util.PersistenceManagerFactoryHelper;
import com.corejdo.examples.model.Author;

/**
 * <p>This simple example shows that no matter how a reference to
 * the same persistent object is made within the context of a given
 * <code>PersistenceManager</code> instance there will only ever be
 * one Java object in-memory.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.
 * There should be no exisiting <code>Author</code> instances in the 
 * datastore for this example to work correctly.</p>
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 *
 */
public final class UniquingExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();
    
    tx.begin();
    
    /*
     * Create a new Author instance
     */
    Author author1 = new Author("Keiron McCammon");
    
    pm.makePersistent(author1);
    
    tx.commit();
    
    tx.begin();
    
    /*
     * Find the previously created Author instance using a Query
     */
    Query query =
      pm.newQuery(Author.class, "name == \"Keiron McCammon\"");
    
    Collection authors = (Collection) query.execute();
    
    Author author2 = (Author) authors.iterator().next();
    
    query.close(authors);
    
    tx.commit();

    /*
     * Validate that the two references actually reference the same
     * in-memory Java object
     */
    if (author1 == author2)
      System.out.println("There is only one object in memory");
    
    pm.close();
  }
  
  private UniquingExample() {}
}
