package com.corejdo.examples.chapter5;

import java.util.*;
import javax.jdo.*;

import com.corejdo.examples.util.*;
import com.corejdo.examples.model.*;

/**
 * <p>This example shows how to use the
 * <code>PersistenceManager.deletePersistent()</code> method to
 * delete persistent objects.</p>
 * <p>It gets an <code>Extent</code> for the <code>Author</code> class from
 * which it gets the first <code>Author</code> instance. It then deletes the
 * author's <code>Address</code> instance and removes each <code>Book<code>
 * instance from the list of book's authored by the author. It then deletes
 * the <code>Author</code> instance.</p>
 * <p>It is not actually necessary to explicit manage the author's associated
 * <code>Address</code> and <code>Book</code> instances in this way since
 * the <code>Author</code> class implements the
 * <code>InstanceCallbacks.jdoPreDelete()</code> method which does this
 * implicitly.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.
 * There should also be at least one <code>Author</code> instance in the database.
 * After the example has been run use the appropriate database browsing tools
 * to confirm the <code>Author</code> instance has been deleted.</p> 
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 */
public class DeletePersistentExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();

    tx.begin();

    Extent extent = pm.getExtent(Author.class, true);

    Iterator authors = extent.iterator();

    if (authors.hasNext()) {

      Author author = (Author) authors.next();

      pm.deletePersistent(author.getAddress());

      Iterator books = author.getBooks().iterator();

      while (books.hasNext()) {

        Book book = (Book) books.next();

        author.removeBook(book);
      }

      pm.deletePersistent(author);
    }

    extent.close(authors);

    tx.commit();

    pm.close();
  }

  private DeletePersistentExample() {}
}
