package com.corejdo.examples.chapter5;

import java.util.*;
import javax.jdo.*;
import com.corejdo.examples.model.*;
import com.corejdo.examples.util.*;

/**
 * <p>This example shows how to use the
 * <code>PersistenceManager.makeTransient()</code> method to make an
 * in-memory persistent object transient so it can be accessed as a normal Java
 * object.</p>
 * <p>It first creates <code>Author</code> instance, committing the transaction.
 * In another transaction it uses the <code>PersistenceManager.retrieve()</code>
 * method to ensure all the fields of the instance have been retrieved from the
 * datastore and then makes it transient. It ends the transaction and shows how
 * the <code>Author</code> instance can still be accessed as a transient 
 * instance.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.
 * There should be at least one <code>Author</code> instance in the database.</p>
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 *
 */
public class MakeTransientExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();
    
    Transaction tx = pm.currentTransaction();
    
    tx.begin();
    
    Author author = new Author("Keiron McCammon");
    
    pm.makePersistent(author);
    
    tx.commit();

    tx.begin();    

    pm.retrieve(author);
    
    pm.makeTransient(author);
    
    tx.commit();
    
    pm.close();
    
    System.out.println("Author's name is: " + author.getName());
  }
  
  private MakeTransientExample() {}
}
