package com.corejdo.examples.chapter5;

import java.util.*;
import javax.jdo.*;

import com.corejdo.examples.util.*;
import com.corejdo.examples.model.*;

/**
 * <p>This example shows how persistent objects can be retrieved from the
 * datastore outside of a transaction using the non-transactional read optional
 * feature.</p>
 * <p>It first uses the <code>Transaction.setNontransactionalRead()</code> method
 * to set the non-transactional read property to true. It then creates an
 * <code>Author</code> instance and commits the transaction, at which point the
 * new instance transitions to being hollow. The author's name is then printed
 * outside of a transaction. Because non-transactional read is enabled, the
 * <code>Author</code> instance is retrieved from the datastore and transitions
 * to being non-transactional. The <code>JDOHelper.isTransactional()</code> method
 * is used to confirm the instance is non-transactional.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.
 * There should be at least one <code>Author</code> instance in the database.</p>
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 *
 */
public class NonTransactionalReadExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();
    
    tx.setNontransactionalRead(true);
    
    tx.begin();
    
    Author author = new Author("Keiron McCammon");
    
    pm.makePersistent(author);
    
    tx.commit();
    
    // Author instance transitions to being hollow after commit
    
    System.out.println("Author's name is '" + author.getName() + "'.");
    
    // Author instance is retrieved from the datastore and
    // transitions to being non-transactional
    
    if (!JDOHelper.isTransactional(author)) {
      
      System.out.println("Author is non-transactional.");
    }
 
    pm.close();
  }
  
  private NonTransactionalReadExample() {}
}
