package com.corejdo.examples.chapter5;

import javax.jdo.*;
import com.corejdo.examples.model.*;
import com.corejdo.examples.util.*;

/**
 * <p>This example shows how the JDO object identity for a persistent object
 * can be used to retrieve the persistent object again.</p>
 * <p>It creates an <code>Author</code> instance, commiting the transaction. It
 * then gets the object identity of the instance and closes the first 
 * <code>PersistenceManager</code> instance. It then gets another
 * <code>PersistenceManager</code> instance and uses the object identity to 
 * retrieve the <code>Author</code> instance again.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.</p>
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 *
 */
public class ObjectIdentityExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();
    
    Transaction tx = pm.currentTransaction();
    
    tx.begin();
    
    Author author = new Author("Keiron McCammon");
    
    pm.makePersistent(author);
    
    tx.commit();
    
    Object oid = pm.getObjectId(author);
    
    System.out.println("Author's object identity is: " + oid); 
    
    pm.close();
    
    PersistenceManager pm2 = pmf.getPersistenceManager();
    
    Transaction tx2 = pm2.currentTransaction();
    
    tx2.begin();
    
    Author author2 = (Author) pm.getObjectById(oid, true);
    
    System.out.println("Author is: " + author2.getName());
    
    tx2.commit();
    
    pm2.close();
  }
  
  private ObjectIdentityExample() {}
}
