package com.corejdo.examples.chapter5;

import java.util.*;
import javax.jdo.*;

import com.corejdo.examples.util.*;
import com.corejdo.examples.model.*;

/**
 * <p>This examples shows how to use optimistic transactions and how
 * conflicting updates are managed.</p>
 * <p>First it creates an <code>Author</code> instance, committing the transaction.
 * Then it uses the <code>Transaction.setOptimisitic()</code> method
 * to set the Optimistic property for the transaction to true. It then gets the
 * author's address and updates the zipcode, thus modifying the <code>Address</code>
 * instance. To ensure that no other transaction has made any changes not just to
 * the <code>Address</code> instance but also the <code>Author</code> instance the
 * <code>PersistenceManager.makeTransactional()</code> method is used to make the 
 * <code>Author</code> instance transactional. Before the transaction is committed
 * a new <code>PersistenceManager</code> instance is used to update the name of the
 * author and commit this change. Now when the first transaction tries to commit
 * a conflicting update is detected and an exception is thrown.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> and <code>Address</code>
 * classes defined.<p>
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Address
 * @see com.corejdo.examples.model.Author
 *
 */
public class OptimisticExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();

    tx.begin();

    Author author =
      new Author(
        "Keiron McCammon",
        new Address("6539 Dumbarton Circle", "Fremont", "CA", "94555"));

    pm.makePersistent(author);

    tx.commit();

    tx.setOptimistic(true);

    tx.begin();

    Address address = author.getAddress();

    address.setZipcode("12345");

    pm.makeTransactional(author);

    /*
     * Another PersistenceManager instance is used to modify
     * the authors name in a different transaction.
     */
    PersistenceManager pm2 = pmf.getPersistenceManager();

    Transaction tx2 = pm.currentTransaction();

    tx2.begin();

    /*
     * The object identity of the Author is used to retrieve
     * the Author instance within the context of the new
     * PersistenceManager instance.
     */
    Author author2 =
      (Author) pm.getObjectById(JDOHelper.getObjectId(author), false);

    author.setName("Sameer Tyagi");

    tx2.commit();

    pm2.close();

    /*
     * Because the Author's name was updated in a different
     * transaction the optimistic transaction will throw
     * an exception on commit.
     */

    try {

      tx.commit();
    }

    /*
     * As of JDO 1.0.1 the exception thrown would be
     * JDOOptimisticVerificationException, prior to this release it
     * would be JDOUserException
     */
    catch (JDOException e) {

      System.out.println("Transaction failed.");
    }

    pm.close();
  }

  private OptimisticExample() {}
}