package com.corejdo.examples.chapter5;

import java.util.Properties;
import java.io.*;
import javax.jdo.*;

/**
 * <p>This example shows how to create a singleton class to manage the
 * explicit creation of a PersistenceManagerFactory instance from a properties
 * file.</p>
 * <p> The <code>getPersistenceManagerFactory()</code> method creates a
 * <code>PersistenceManagerFactory</code> instance using the
 * <code>JDOHelper.getPersistenceManagerFactory()</code> method. It creates a set of
 * properties from a properties file whose name is specified using the system
 * property "jdo.properties". If the property is not defined and cannot be accessed
 * then <code>JDOFatalUserException</code> is thrown.</p>
 *
 * @author mccammon
 *
 */
public class PMFSingletonFromFileExample {

  // PersistenceManagerFactory singleton
  private static PersistenceManagerFactory pmf;

  /**
   * <p>This method returns the singleton
   * PersistenceManagerFactory. If the singleton 
   * hasn't been initialized it creates a PersistenceManagerFactory
   * from a properties file denoted by the system property "jdo.properties"
   * and initializes the singleton.</p>
   * 
   * @throws JDOFatalUserException if an error occured creating
   * the PersistenceManagerFactory instance
   * 
   * @return the PersistenceManagerFactory singleton
   */
  public static PersistenceManagerFactory getPersistenceManagerFactory() {

    if (pmf == null) {

      String filename = System.getProperty("jdo.properties");

      if (filename == null) {

        throw new JDOFatalUserException("System property 'jdo.properties' not defined");
      }

      else {

        Properties properties = new Properties();

        try {

          properties.load(new FileInputStream(filename));

          pmf = JDOHelper.getPersistenceManagerFactory(properties);
        }

        catch (java.io.IOException e) {

          throw new JDOFatalUserException(
            "Error reading '" + filename + "'",
            e);
        }
      }
    }

    return pmf;
  }
  
  private PMFSingletonFromFileExample() {}
}
