package com.corejdo.examples.chapter5;

import java.util.Properties;
import javax.naming.*;
import javax.jdo.*;

/**
 * <p>This example shows how to create a singleton class to manage the
 * lookup of a <code>PersistenceManagerFactory</code> instance via JNDI.</p>
 * <p>The <code>getPersistenceManagerFactory()</code> method looks up
 * the instance via JNDI using its class name. If this fails it throws
 * <code>JDOFatalUserException</code>.</p>
 * <p>This class can be run to explicitly create a
 * <code>PersistenceManagerFactory</code> instance and bind it to its JNDI name.</p>
 * 
 * @author mccammon
 *
 */
public class PMFSingletonFromJNDIExample {
  
  // JNDI name for the PersistenceManagerFactory instance
  private static String name = PMFSingletonFromJNDIExample.class.getName();
  
  // PersistenceManagerFactory singleton
  private static PersistenceManagerFactory pmf;
  
  /**
   * <p>This method returns the singleton
   * PersistenceManagerFactory. If required it first
   * initializes the singleton by looking it up via JNDI.
   * If it can't lookup the PersistenceManagerFactory via 
   * JNDI it throws a JDOFatalUserException</p>
   * 
   * @return the PersistenceManagerFactory instance
   */
  public static PersistenceManagerFactory getPersistenceManagerFactory() {
    
    if (pmf == null) {
      
      try {
  
        InitialContext context = new InitialContext();
  
        pmf = (PersistenceManagerFactory) context.lookup(name);
      }
      
      catch (NamingException e) {
  
        throw new JDOFatalUserException(
          "Can't lookup PersistenceManagerFactory: " + name,
          e);
      }
    }
    
    return pmf;
  }

  /**
   * <p>This class can be run to create a
   * PersistenceManagerFactory instance and register it with JNDI.
   * The PersistenceManagerFactoryClass property needs to be specified.</p>
   *
   * @param args command line arguments: url name password
   */
  public static void main(String[] args) {

    if (args.length != 3) {
      
      System.out.println("Invalid arguments, use: <url> <name> <password>");
      
      System.exit(-1);
    }
    
    Properties properties = new Properties();
    
    properties.put(
      "javax.jdo.PersistenceManagerFactoryClass", "XXX");
    properties.put(
      "javax.jdo.option.ConnectionURL", args[0]);
    properties.put(
      "javax.jdo.option.ConnectionUserName", args[1]);
    properties.put(
      "javax.jdo.option.ConnectionPassword", args[2]);

    PersistenceManagerFactory pmf =
      JDOHelper.getPersistenceManagerFactory(properties);

    try {

      InitialContext context = new InitialContext();

      context.bind(name, pmf);
    }
    
    catch (NamingException e) {

      System.out.println("Can't bind PersistenceManager");
      
      e.printStackTrace();
    }
  }
  
  private PMFSingletonFromJNDIExample() {}
}
