package com.corejdo.examples.chapter5;

import java.util.*;
import javax.jdo.*;
import com.corejdo.examples.model.*;
import com.corejdo.examples.util.*;

/**
 * <p>This example shows how a persistent object can be made non-transactional
 * and accessed outside of a transaction and even modified.</p>
 * <p>First an <code>Author</code> instance is created, committing the transaction.
 * The instance is then made non-transactional using the 
 * <code>PersistenceManager.makeNontransactional()</code> method. The name of the
 * author is printed to show it can be accessed outside of a transaction. Next
 * the author's name is modified and again printed to show it has been changed.
 * Finally the instance is made transactional again using the
 * <code>PersistenceManager.makeTransactional()</code> method.
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.<p>
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 *
 */
public class PersistentTransactionalExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();

    tx.begin();
    
    Author author = new Author("Keiron McCammon");
    
    pm.makePersistent(author);
    
    tx.commit();
    
    System.out.println("Making instance non-transactional.");
    
    pm.makeNontransactional(author);
    
    System.out.println("Author's name is '" + author.getName() + "'");
    
    author.setName("Sameer Tyagi");
    
    System.out.println("Modified name is '" + author.getName() + "'");
    
    tx.begin();
    
    System.out.println("Making instance transactional.");
    
    pm.makeTransactional(author);
    
    System.out.println("Transactional name is '" + author.getName() + "'");
    
    tx.commit();

    pm.close();
  }
  
  private PersistentTransactionalExample() {}
}
