package com.corejdo.examples.chapter5;

import java.util.*;
import javax.jdo.*;

import com.corejdo.examples.util.*;
import com.corejdo.examples.model.*;

/**
 * <p>This example shows how the <code>RestoreValues<code> transaction property
 * works in conjunction with the <code>RetainValues</code> property.</p>
 * <p>It first uses the <code>Transaction.setRetainValues()</code> method to 
 * set the <code>RetainValues</code> property to true for the transaction.
 * It then creates an <code>Author</code> instance, committing the transaction. It then
 * uses the <code>JDOHelper.isTransactional()</code> method to show the
 * <code>Author</code> instance has transitioned to being non-transactional at the
 * end of the transaction as a result of the <code>RetainValues</code> property.
 * In a new transaction the author's name is changed but the transaction is 
 * rolled back, the author's name is printed to see that the changed name remains
 * in-memory. Then the <code>Transaction.setRestoreValues()</code> method is
 * used to set the <code>RestoreValues</code> property to true for the
 * transaction. This time when the author's name is changed and the transaction
 * rolled back the <code>Author</code> instance reverts to the unchanged name
 * as a result of the <code>RestoreValues</code> property.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.</p> 
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 *
 */
public class RestoreValuesExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();
    
    tx.setRetainValues(true);
    
    tx.begin();
    
    Author author = new Author("Keiron McCammon");
    
    pm.makePersistent(author);
    
    tx.commit();
    
    /*
     * Author instance is retained after commit and transitions
     * to being non-transactional.
     */
    if (!JDOHelper.isTransactional(author)) {
      
      System.out.println("Author is non-transactional.");
    }
    
    System.out.println("Author's name is '" + author.getName() + "'.");
    
    tx.begin();
    
    author.setName("Sameer Tyagi");
    
    tx.rollback();
    
    /*
     * Transaction was rolled back rather than committed
     * therefore the name change is not in the datastore.
     * However, the changed name is retained in-memory.
     */
    System.out.println(
      "Author's name is '" + author.getName() + "' after rollback.");
    
    tx.setRestoreValues(true);
    
    tx.begin();
    
    author.setName("Sameer Tyagi");
    
    tx.rollback();
    
    /*
     * This time because RestoreValues is true the authors
     * name is reverted back to its original name on rollback
     */
    System.out.println(
      "Author's name is '" + author.getName() + 
      "' after rollback with RestoreValues.");
    
    pm.close();
  }
  
  private RestoreValuesExample() {}
}
