package com.corejdo.examples.chapter5;

import java.util.*;
import javax.jdo.*;

import com.corejdo.examples.util.*;
import com.corejdo.examples.model.*;

/**
 * <p>This example shows how the <code>RetainValues</code> transaction property
 * works.</p>
 * <p>It first uses the <code>Transaction.setRetainValues()</code> method to 
 * set the <code>RetainValues</code> property to true for the transaction. It then
 * creates an <code>Author</code> instance, committing the transaction. It then
 * uses the <code>JDOHelper.isTransactional()</code> method to show the
 * <code>Author</code> instance has transitioned to being non-transactional at the
 * end of the transaction as a result of the <code>RetainValues</code> property.
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.</p> 
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 *
 */
public class RetainValuesExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();
    
    tx.setRetainValues(true);
    
    tx.begin();
    
    Author author = new Author("Keiron McCammon");
    
    pm.makePersistent(author);
    
    tx.commit();
    
    /*
     * Author instance transitions to being non-transactional
     * after commit and its fields remain in-memory
     */
    if (!JDOHelper.isTransactional(author)) {
      
      System.out.println("Author is non-transactional.");
    }
    
    System.out.println("Author's name is '" + author.getName() + "'.");
    
    pm.close();
  }
  
  private RetainValuesExample() {}
}
