package com.corejdo.examples.chapter5;

import java.util.*;
import javax.jdo.*;

import com.corejdo.examples.util.*;
import com.corejdo.examples.model.*;

/**
 * <p>This example shows how the <code>PersistenceManager.retrieveAll()</code>
 * method can be used to retrieve groups of persistent objects from the datastore
 * in one go.</p>
 * <p>It gets an <code>Extent</code> for the <code>Book</code> class and iterates
 * through it getting 1024 instances at a time. It then uses the 
 * <code>PersistenceManager.retrieveAll()</code> method to retrieve 1024 instances
 * and then prints the title of each. It uses the 
 * <code>PersistenceManager.evictAll()</code> method to evict the instances once
 * finished with them.</p>
 * <p>This demonstrates a performance optimization of retrieving persistent objects
 * in groups rather than one by one.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Book</code> class defined.</p> 
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Book
 *
 */
public class RetrieveExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();
    
    tx.begin();
    
    Extent extent = pm.getExtent(Book.class, true);

    Iterator books = extent.iterator();
    
    System.out.println("Listing of all books:");
    
    List list = new ArrayList(1024);
          
    while (books.hasNext()) {
    
      list.add(books.next());
      
      if (list.size() == 1024 || !books.hasNext()) {
    
        pm.retrieveAll(list);
        
        for (int i = 0; i < list.size(); ++i) {
                      
          Book book = (Book) list.get(i);
    
          System.out.println("  " + book.getTitle());
        }
      
        pm.evictAll(list);      
        
        list.clear();
      }
    }

    extent.close(books);
        
    tx.commit();
    
    pm.close();
  }
  
  private RetrieveExample() {}
}
