package com.corejdo.examples.chapter5;

import java.util.*;
import javax.jdo.*;

import com.corejdo.examples.util.*;
import com.corejdo.examples.model.*;

/**
 * <p>This example shows how to use the <code>contains()</code> method in a query.</p>
 * <p>It finds all <code>Book</code> instances authored by an <code>Author</code>
 * with a specified name. Where the <code>Book</code> class has a set of
 * <code>Author</code> instances.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> and
 * <code>Book</code> classes defined.</p>
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 * @see com.corejdo.examples.model.Book
 * 
 */
public class SimpleQueryUsingContainsExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();

    tx.begin();

    Author author = new Author("Keiron McCammon");

    author.addBook(new Book("Core Java Data Objects", "0-13-140731-7"));

    pm.makePersistent(author);

    tx.commit();

    tx.begin();

    Query query =
      pm.newQuery(
        Book.class,
        "authors.contains(author) && author.name == name");

    query.declareParameters("String name");
    query.declareVariables("Author author");

    Collection result = (Collection) query.execute("Keiron McCammon");

    Iterator iter = result.iterator();

    while (iter.hasNext()) {

      Book book = (Book) iter.next();

      System.out.println(book.getTitle() + " - " + book.getISBN());
    }

    query.close(result);

    tx.commit();

    pm.close();
  }

  private SimpleQueryUsingContainsExample() {}
}
