package com.corejdo.examples.chapter5;

import java.util.*;
import javax.jdo.*;

import com.corejdo.examples.util.*;
import com.corejdo.examples.model.*;

/**
 * <p>This example shows how to use navigation in a query.</p>
 * <p>It finds all <code>Book</code> instances published by a <code>Publisher</code>
 * with a specified name. Where the <code>Book</code> class has a reference to a
 * <code>Publisher</code> instance.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Publisher</code> and
 * <code>Book</code> classes defined.</p>
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Book
 * @see com.corejdo.examples.model.Publisher
 *
 */
public class SimpleQueryWithNavigationExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();
    
    tx.begin();

    Publisher publisher = new Publisher("Sun Microsystems Press");

    publisher.addBook(new Book("Core Java Data Objects", "0-13-140731-7"));

    pm.makePersistent(publisher);

    tx.commit();
    
    tx.begin();

    Query query = pm.newQuery(Book.class, "publisher.name == name");    
       
    query.declareParameters("String name");
    
    Collection result = (Collection) query.execute("Sun Microsystems Press");
    
    Iterator iter = result.iterator();
    
    while (iter.hasNext()) {
      
      Book book = (Book) iter.next();
      
      System.out.println(book.getTitle() + " - " + book.getISBN());
    }
    
    query.close(result);
    
    tx.commit();
    
    pm.close();
  }
  
  private SimpleQueryWithNavigationExample() {}  
}
