package com.corejdo.examples.chapter5;

import javax.jdo.*;
import com.corejdo.examples.model.*;
import com.corejdo.examples.util.*;

/**
 * <p>This example shows how the JDO object identity for a persistent object
 * can be converted to a string and then converted back to an object identity
 * sometime later and used to retrieve the persistent object again.</p>
 * <p>It first creates an <code>Author</code> instance, committing the transaction.
 * It then gets the object identity of the instance as a string and closes the
 * <code>PersistentManager</code> instance. It then gets another
 * <code>PersistentManager</code> instance and uses the
 * <code>PersistentManager.newObjectIdInstance()</code> method to convert the string
 * back to an object identity. It then uses the
 * <code>PersistentManager.getObjectById()</code> method to get the persistent
 * object again.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.</p> 
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 *
 */
public class StringObjectIdentityExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Transaction tx = pm.currentTransaction();
    
    tx.begin();
    
    Author author = new Author("Keiron McCammon");
    
    pm.makePersistent(author);
    
    tx.commit();
    
    String oid = pm.getObjectId(author).toString();
    
    System.out.println("Author's object identity is: " + oid);
    
    pm.close();
    
    PersistenceManager pm2 = pmf.getPersistenceManager();
    
    Transaction tx2 = pm2.currentTransaction();
    
    tx2.begin();
    
    Author author2 =
      (Author) pm.getObjectById(
        pm.newObjectIdInstance(Author.class, oid),
        true);
    
    System.out.println("Author is: " + author2.getName());
    
    tx2.commit();

    pm2.close();
  }
}
