package com.corejdo.examples.chapter5;

import java.util.*;
import javax.jdo.*;
import com.corejdo.examples.model.*;
import com.corejdo.examples.util.*;

/**
 * <p>This example shows how a transient instance of a persistence capable
 * class can be made transactional.</p>
 * <p>It first creates a transient <code>Author</code> instance. It uses the
 * <code>PersistenceManager.makeTransactional()</code> method to make it
 * transactional. Next a transaction is started and the author's name is changed,
 * however the transaction is rolled back. Because the transient <code>Author</code>
 * instance was made transactional the name change is also rolled back. Now
 * the instance is made non-transactional using the
 * <code>PersistenceManager.makeNontransactional()</code> method. This time when
 * the author's name is modified and the transaction rolled back the changed name
 * remains.</p>
 * <p>It gets a <code>PersistenceManagerFactory</code> instance via the
 * {@link com.corejdo.examples.util.PersistenceManagerFactoryHelper} class,
 * therefore <code>jdo.properties</code> system property should be set 
 * to a valid properties file for the JDO implementation being used.</p>
 * <p>Before this example can be run a database should be created and
 * if necessary the schema for the <code>Author</code> class defined.</p> 
 * 
 * @author mccammon
 * @see com.corejdo.examples.util.PersistenceManagerFactoryHelper
 * @see com.corejdo.examples.model.Author
 *
 */
public class TransientTransactionalExample {

  /**
   * <p>Used to run the example. No command line arguments are required.</p>
   * 
   * @param args command line arguments
   */
  public static void main(String[] args) {

    PersistenceManagerFactory pmf =
      PersistenceManagerFactoryHelper.getPersistenceManagerFactory();

    PersistenceManager pm = pmf.getPersistenceManager();

    Author author = new Author("Keiron McCammon");
    
    System.out.println("Making instance transactional.");
    
    pm.makeTransactional(author);
    
    Transaction tx = pm.currentTransaction();
    
    tx.begin();
    
    System.out.println("Author's name is '" + author.getName() + "'");
    
    author.setName("Sameer Tyagi");
    
    System.out.println("Modified name is '" + author.getName() + "'");
    
    tx.rollback();
    
    System.out.println(
      "Name after rollback is '" + author.getName() + "'");
    
    System.out.println("Making instance non-transactional.");
    
    pm.makeNontransactional(author);
    
    tx.begin();
    
    System.out.println("Name is '" + author.getName() + "'");
    
    author.setName("Sameer Tyagi");
    
    System.out.println("Modified name is '" + author.getName() + "'");
    
    tx.rollback();
    
    System.out.println(
      "Name after rollback is '" + author.getName() + "'");

    pm.close();
  }
  
  private TransientTransactionalExample() {}
}
