package com.corejdo.examples.model;

import java.util.*;
import javax.jdo.*;

/**
 * <p>This class represents an author.</p>
 * <p>An author has a name, an address and a collection of books that he/she has
 * written. The name and address can be accessed using JavaBeans style
 * getters/setters. The collection of books written by an author should be 
 * manipulated using the {@link #getBooks getBooks()}, {@link #addBook addBook()}
 * and {@link #removeBook removeBook()} methods. When a Book is added to an
 * Author it implicitly adds the Author to the list of author's for the book
 * also.</p>
 * <p>It uses the jdoPreDelete() callback to implicitly manage associated instances
 * on deletion.</p>
 * 
 * @author mccammon
 *
 */
public class Author implements InstanceCallbacks {

  protected String name;
  protected Address address;
  protected Set books;

  /* An empty array that is returned by the findByName() method */ 
  private final static Author[] EMPTY_ARRAY = new Author[0];

  /**
   * <p>Finds all author's with the specified name.</p>
   * <p>It uses the provided PersistenceManager instance to query the datastore
   * using the specified author's name. It returns an array of Author instances
   * that match the specified name. If there are no matches then an empty array is
   * returned.</p>
   * 
   * @param pm the PersistenceManager factory to be used for the query
   * @param name the name of the authors to be found
   * @return an array of Author instances with the given name, if there are no
   * matching Author instances then an empty array is returned.
   */
  public static Author[] findByName(PersistenceManager pm, String name) {

    Author[] authors = EMPTY_ARRAY;

    Query query = pm.newQuery(Author.class, "this.name == name");

    query.declareParameters("String name");

    Collection result = (Collection) query.execute(name);

    authors = (Author[]) result.toArray(authors);

    query.close(result);

    return authors;
  }

  /**
   * <p>Creates an Author instance with the specified name and address.</p>
   * 
   * @param name the name
   * @param address the address
   */
  public Author(String name, Address address) {

    this.name = name;
    this.address = address;
  }
  
  /**
   * <p>Creates an Author instance with the specified name.</p>
   * 
   * @param name the name
   */
  public Author(String name) {
    
    this.name = name;
  }

  protected Author() {}

  /**
   * <p>Gets the name of the author.</p>
   * 
   * @return the name
   */
  public String getName() {

    return name;
  }

  /**
   * <p>Sets the name of the author.</p>
   * 
   * @param name the name
   */
  public void setName(String name) {

    this.name = name;
  }

  /**
   * <p>Gets the Address instance that represents the address of the author.</p>
   * 
   * @return the Address instance, or null if no address has been specified for
   * the Author
   */
  public Address getAddress() {

    return address;
  }

  /**
   * <p>Gets the collection of books written by the author.</p>
   * <p>It returns a unmodifiable collection, the collection of Book instances
   * written by an author should be changed using the {@link #addBook addBook()} and
   * {@link #removeBook removeBook()} methods.</p>
   * 
   * @return an unmodifiable collection of Book instances, if the Author has not
   * written any books an empty collection is returned.
   */
  public Collection getBooks() {

    return (
      books != null
        ? Collections.unmodifiableSet(books)
        : Collections.EMPTY_SET);
  }

  /**
   * <p>Adds a book to the collection of books written by the author.</p>
   * <p>The author will implicitly be added to the list of author's for the
   * book.</p>
   * 
   * @param book the book
   */
  public void addBook(Book book) {

    if (books == null) {

      books = new HashSet();
    }

    books.add(book);

    book.addAuthor(this);
  }

  /**
   * <p>Removes a book from the collection of books written by the author.</p>
   * <p>The author will implicitly be removed from the list of author's for the
   * book.</p>
   * 
   * @param book the book
   */
  public void removeBook(Book book) {

    if (books != null) {

      books.remove(book);

      book.removeAuthor(this);
      
      if (books.isEmpty()) {
        
        books = null;
      }
    }
  }

  /**
   * <p>Not used.</p>
   * 
   * @see javax.jdo.InstanceCallbacks#jdoPostLoad()
   */
  public void jdoPostLoad() {}

  /**
   * <p>Not used.</p>
   * 
   * @see javax.jdo.InstanceCallbacks#jdoPreClear()
   */
  public void jdoPreClear() {}

  /**
   * <p>This callback is used to implicitly manage associates with other instances
   * upon deletion.</p>
   * <p>It deletes the author's associated Address instance and removes the author
   * from the list of author's of each book in the author's collection of
   * authored books.</p>
   * 
   * @see javax.jdo.InstanceCallbacks#jdoPreDelete()
   */
  public void jdoPreDelete() {

    JDOHelper.getPersistenceManager(this).deletePersistent(address);

    if (books != null) {

      Iterator iter = books.iterator();

      while (iter.hasNext()) {

        Book book = (Book) iter.next();

        book.removeAuthor(this);
      }
      
      books = null;
    }
  }

  /**
   * <p>Not used.</p>
   * 
   * @see javax.jdo.InstanceCallbacks#jdoPreStore()
   */
  public void jdoPreStore() {}
}
