package com.corejdo.examples.model;

import java.util.*;
import javax.jdo.*;

/**
 * <p>A book has a title, an ISBN number and a publication date. It has a
 * collection of author's that wrote the book and a publisher that published it.
 * The title, ISBN and publication date can be accessed using
 * JavaBeans style getters/setters. The collection of authors can be accessed using
 * the {@link #getAuthors getAuthors()} method, however it is not possible to
 * explicitly add an author to this collection. Instead a book is added to an
 * author, which implicitly adds the author to the set of authors for the book.
 * Likewise the publisher can be accessed using the
 * {@link #getPublisher getPublisher()} method, however it is not possible to
 * explicitly set the publisher. Instead a book is added to a publisher,
 * which implicitly sets the publisher for the book.</p>
 * <p>It uses the jdoPreDelete() callback to implicitly manage associated instances
 * on deletion.</p>
 * 
 * @author mccammon
 *
 */
public class Book implements InstanceCallbacks {

  private String title;
  private String isbn;
  private Date publicationDate;
  private Set authors;
  private Publisher publisher;

  /* An empty array that is returned by the findByTitle() method */ 
  private final static Book[] EMPTY_ARRAY = new Book[0];
  
  /**
   * <p>Finds all books's with the specified title.</p>
   * <p>It uses the provided PersistenceManager instance to query the datastore
   * using the specified book's title. It returns an array of Book instances
   * that match the specified title. If there are no matches then an empty array is
   * returned.</p>
   * 
   * @param pm the PersistenceManager factory to be used for the query
   * @param title the title of the books to be found
   * @return an array of Book instances with the given title, if there are no
   * matching Book instances then an empty array is returned.
   */
  public static Book[] findByTitle(PersistenceManager pm, String title) {

    Book[] books = EMPTY_ARRAY;

    Query query = pm.newQuery(Book.class, "this.title == title");

    query.declareParameters("String title");

    Collection result = (Collection) query.execute(title);

    books = (Book[]) result.toArray(books);
    
    query.close(result);

    return books;
  }

  /**
   * <p>Creates a Book instance with the specified title, isbn number
   * and publication date.</p>
   * <p>Once created a book should be added to one or more authors and a
   * publisher.</p>
   * 
   * @param title the title
   * @param isbn the isbn number
   * @param publicationDate the publication date
   */
  public Book(
    String title,
    String isbn,
    Date publicationDate) {

    this.title = title;
    this.isbn = isbn;
    this.publicationDate = publicationDate;
  }

  /**
   * <p>Creates a Book instance with the specified title and isbn number.</p>
   * <p>Once created a book should be added to one or more authors and a
   * publisher.</p>
   * 
   * @param title the title
   * @param isbn the isbn number
   */
  public Book(String title, String isbn) {
    
    this.title = title;
    this.isbn = isbn;
  }
  
  protected Book() {}

  /**
   * <p>Gets the collection of authors that wrote by the book.</p>
   * <p>It returns a unmodifiable collection.</p>
   * 
   * @return an unmodifiable collection of Author instances, if the Book has not
   * yet got any authors an empty collection is returned.
   */
  public Collection getAuthors() {

    return (
      authors != null
        ? Collections.unmodifiableSet(authors)
        : Collections.EMPTY_SET);
  }

  /**
   * <p>Gets the ISBN number.</p>
   * 
   * @return the ISBN number
   */
  public String getISBN() {

    return isbn;
  }

  /**
   * <p>Sets the ISBN number.</p>
   * 
   * @param isbn the ISBN number
   */
  public void setISBN(String isbn) {

    this.isbn = isbn;
  }

  /**
   * <p>Gets the publication date.</p>
   * 
   * @return the publication date
   */
  public Date getPublicationDate() {

    return publicationDate;
  }

  /**
   * <p>Sets the publication date.</p>
   * 
   * @param publicationDate the publication date
   */
  public void setPublicationDate(Date publicationDate) {

    this.publicationDate = publicationDate;
  }

  /**
   * <p>Gets the title.</p>
   * 
   * @return the title
   */
  public String getTitle() {

    return title;
  }

  /**
   * <p>Sets the title.<p>
   * 
   * @param title the title
   */
  public void setTitle(String title) {

    this.title = title;
  }

  /**
   * <p>Gets the publisher.</p>
   * 
   * @return the publisher, or null if the book does not yet have a publisher
   */
  public Publisher getPublisher() {

    return publisher;
  }

  /**
   * <p>Sets the publisher.</p>
   * <p>If there is already a publisher the book is removed from the collection
   * of books published by that publisher first.</p>
   * 
   * @param publiser the publisher
   */
  void setPublisher(Publisher publisher) {

    this.publisher = publisher;
  }

  /**
   * <p>Adds an author to the collection of authors for the book.</p>
   * 
   * @param author the author
   */
  void addAuthor(Author author) {

    if (authors == null) {
      
      authors = new HashSet();
    }
    
    authors.add(author);
  }

  /**
   * <p>Removes an author from the collection of authors for the book.</p>
   * 
   * @param author the author
   */
  void removeAuthor(Author author) {

    if (authors != null) {
      
      authors.remove(author);
      
      if (authors.isEmpty()) {
        
        authors = null;
      }
    }    
  }
  
  /**
   * <p>Not used.</p>
   * 
   * @see javax.jdo.InstanceCallbacks#jdoPostLoad()
   */
  public void jdoPostLoad() {}

  /**
   * <p>Not used.</p>
   * 
   * @see javax.jdo.InstanceCallbacks#jdoPreClear()
   */
  public void jdoPreClear() {}

  /**
   * <p>This callback is used to implicitly manage associates with other instances
   * upon deletion.</p>
   * <p>It ensures that the book is removed from the set of books for each author
   * in the collection of authors for the book.</p>
   * 
   * @see javax.jdo.InstanceCallbacks#jdoPreDelete()
   */
  public void jdoPreDelete() {
    
    if (authors != null) {
      
      Iterator iter = authors.iterator();
      
      while (iter.hasNext()) {
        
        Author author = (Author) iter.next();
        
        author.removeBook(this);
      }
    }
    
    if (publisher != null) {
      
      publisher.removeBook(this);
    }
  }

  /**
   * <p>Not used.</p>
   * 
   * @see javax.jdo.InstanceCallbacks#jdoPreStore()
   */
  public void jdoPreStore() {}  
}
