package com.corejdo.examples.model;

import java.util.*;
import javax.jdo.*;

/**
 * <p>This class represents a publisher.</p>
 * <p>A publisher has a name and a collection of books that it publishes. The name
 * can be accessed using JavaBeans style getters/setters. The collection of books
 * can be managed using the {@link #getBooks getBooks()},
 * {@link #addBook addBook()} and {@link #removeBook removeBook()} methods.</p>
 * <p>It uses the jdoPreDelete() callback to implicitly manage associated instances
 * on deletion.</p>
 * 
 * @author mccammon
 *
 */
public class Publisher implements InstanceCallbacks {

  private String name;
  private Set books;

  private final static Publisher[] EMPTY_ARRAY = new Publisher[0];
  
  /**
   * <p>Finds all publisher's with the specified name.</p>
   * <p>It uses the provided PersistenceManager instance to query the datastore
   * using the specified publisher's name. It returns an array of Publisher
   * instances that match the specified name. If there are no matches then an
   * empty array is returned.</p>
   * 
   * @param pm the PersistenceManager factory to be used for the query
   * @param name the name of the publisher to be found
   * @return an array of Publisher instances with the given name, if there are no
   * matching Publisher instances then an empty array is returned.
   */
  public static Publisher[] findByName(PersistenceManager pm, String name) {

    Publisher[] publishers = EMPTY_ARRAY;

    Query query = pm.newQuery(Publisher.class, "this.name == name");

    query.declareParameters("String name");

    Collection result = (Collection) query.execute(name);

    publishers = (Publisher[]) result.toArray(publishers);

    query.close(result);

    return publishers;
  }

  /**
   * <p>Creates a publisher with the given name.<p>
   * 
   * @param name the name
   */
  public Publisher(String name) {

    this.name = name;
  }
  
  protected Publisher() {}

  /**
   * <p>Gets the name.</p>
   * 
   * @return the name
   */
  public String getName() {
    
    return name;
  }

  /**
   * <p>Sets the name.</p>
   * 
   * @param name the name
   */
  public void setName(String name) {
    
    this.name = name;
  }

  /**
   * <p>Gets the collection of books published by the publisher.</p>
   * <p>It returns a unmodifiable collection, the collection of Book instances
   * published by an author should be changed using the {@link #addBook} and
   * {@link #removeBook} methods.</p>
   * 
   * @return an unmodifiable collection of Publisher instances, if the Publihser
   * has not yet published any books an empty collection is returned.
   */
  public Collection getBooks() {
    
    return (
      books != null
        ? Collections.unmodifiableSet(books)
        : Collections.EMPTY_SET);
  }

  /**
   * <p>Adds a book to the collection of books published by the publisher.</p>
   * <p>The publisher for the book is implicitly set to this publisher.</p>
   * 
   * @param book the book
   */
  public void addBook(Book book) {
    
    if (books == null) {

      books = new HashSet();
    }

    books.add(book);

    book.setPublisher(this);
  }
  
  /**
   * <p>Removes a book from the collection of books publihsed by the publisher.</p>
   * <p>The publisher for the book is implicitly set to null.</p>
   * 
   * @param book the book
   */
  public void removeBook(Book book) {
   
    if (books != null) {
      
      books.remove(book);
      
      book.setPublisher(null);
      
      if (books.isEmpty()) {
        
        books = null;
      }
    } 
  }
  
  /**
   * <p>Not used.</p>
   * 
   * @see javax.jdo.InstanceCallbacks#jdoPostLoad()
   */
  public void jdoPostLoad() {}

  /**
   * <p>Not used.</p>
   * 
   * @see javax.jdo.InstanceCallbacks#jdoPreClear()
   */
  public void jdoPreClear() {}

  /**
   * @see javax.jdo.InstanceCallbacks#jdoPreDelete()
   */
  public void jdoPreDelete() {

    if (books != null) {

      Iterator iter = books.iterator();
      
      while (iter.hasNext()) {
        
        Book book = (Book) iter.next();
        
        book.setPublisher(null);
      }
      
      books = null;
    }
  }

  /**
   * <p>Not used.</p>
   * 
   * @see javax.jdo.InstanceCallbacks#jdoPreStore()
   */
  public void jdoPreStore() {}
}
