package com.corejdo.examples.util;

import java.util.Properties;
import java.io.*;
import javax.jdo.*;

/**
 * <p>This is a singleton class that can be used to get a
 * <code>PersistenceManagerFactory</code> instance. The instance
 * is created from the properties contained in the file specified
 * by the <code>jdo.properties</code> system property.</p>
 * <p>The property file should at least contain the
 * <code>javax.jdo.PersistenceManagerFactoryClass</code> option to specify
 * the vendor's class name. In addition the following options are also
 * likely to be required: <code>javax.jdo.option.ConnectionURL</code>;
 * <code>javax.jdo.option.UserName</code> and
 * <code>javax.jdo.option.Password</code>.</p>
 * <p>If the system property isn't set or the file can't be read then
 * <code>JDOFatalUserException</code> will be thrown.</p>
 * 
 * @author mccammon
 *
 */
public class PersistenceManagerFactoryHelper {

  // PersistenceManagerFactory singleton
  private static PersistenceManagerFactory pmf;

  /**
   * <p>This method returns the singleton
   * PersistenceManagerFactory. If the singleton 
   * hasn't been initialized it creates a PersistenceManagerFactory
   * from a properties file denoted by the system property "jdo.properties"
   * and initializes the singleton.</p>
   * 
   * @throws JDOFatalUserException if an error occured creating
   * the PersistenceManagerFactory instance
   * 
   * @return the PersistenceManagerFactory singleton
   */
  public static PersistenceManagerFactory getPersistenceManagerFactory() {

    if (pmf == null) {

      String filename = System.getProperty("jdo.properties");

      if (filename == null) {

        throw new JDOFatalUserException("System property 'jdo.properties' not defined");
      }

      Properties properties = new Properties();

      try {

        FileInputStream file = new FileInputStream(filename);

        properties.load(file);

        file.close();

        pmf = JDOHelper.getPersistenceManagerFactory(properties);
      }

      catch (java.io.IOException e) {

        throw new JDOFatalUserException(
          "Error reading '" + filename + "'",
          e);
      }
    }

    return pmf;
  }
  
  private PersistenceManagerFactoryHelper() {}
}
